﻿//---------------------------------------------------------------------------
// 串口控件演示程序，Copyright © Victor Chen

#include <fmx.h>
#pragma hdrstop

#include "UnitTsData.h"
#include "UnitSendFile.h"
#include <System.IOUtils.hpp>
#if(__BORLANDC__>=0x0720) // 0x0720: C++Builder 10.1 Berlin
#  include <Fmx.DialogService.Sync.hpp>
#endif
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "Fmx.VictorFSM"
#pragma link "Fmx.YbCommDevice"
#pragma resource "*.fmx"
TForm1 *Form1;
//---------------------------------------------------------------------------
__fastcall TForm1::TForm1(TComponent* Owner)
    : TForm(Owner)
{
  _hSendFile = INVALID_HANDLE_VALUE;
  _hReceiveFile = INVALID_HANDLE_VALUE;

  _ucPkgID = 0;
  _ucTryTimes = 0;

  _lUploadDataPos = 0;
  _lUploadDataSize = 0;
  _lUploadFileSize = 0;

  _lBytesToReceive = 0;
  _dtReceiveFileTime = 0;

  _sReceivePath = ExtractFileDir(ParamStr(0))+L"\\download";
  TDirectory::CreateDirectory(_sReceivePath);
  _sReceivePath += L"\\";

  YbCommDevice1->PackageType = cptFrameHeadTail;
  YbCommDevice1->PackageSize = 9000; //最大允许发送9000字节的数据包
  YbCommDevice1->UsePackage  = true;

  YbCommDevice1->FrameSettings->FrameHead = 0xdb; //数据包头
  YbCommDevice1->FrameSettings->FrameTail = 0xde; //数据包尾
  YbCommDevice1->FrameSettings->FrameCtrl = 0xdc; //数据控制符

  try
   {
     YbCommDevice1->Active = true;
   }
  catch(Exception &e)
   {
     if(!YbCommDevice1->SettingsDialog(this,true))
       Application->Terminate();
   }
}
//---------------------------------------------------------------------------
void __fastcall TForm1::ShowInfo(UnicodeString s)
{
  Memo1->Lines->Add(FormatDateTime(L"yyyy-mm-dd hh:nn:ss ",Now())+s);
}
//---------------------------------------------------------------------------
void __fastcall TForm1::BtnSetClick(TObject *Sender)
{
  YbCommDevice1->SettingsDialog(this,true);
}
//---------------------------------------------------------------------------
void __fastcall TForm1::BtnSendFileClick(TObject *Sender)
{
  if(VictorFSM1->State != mfsIdle)
   {
     ShowErrDlg(L"正在发送文件，请等待发送完成再进行操作！");
     return;
   }
  if(_bReceivingFile)
   {
     ShowErrDlg(L"正在接收文件，请等待接收完成再进行操作！");
     return;
   }
  if(OpenDialog1->Execute())
   {
     _sUploadFileName = OpenDialog1->FileName;
     VictorFSM1->State = mfsPrepareSendFile; // 准备发送文件
   }
}
//---------------------------------------------------------------------------
void __fastcall TForm1::VictorFSM1ProcessState(TObject *Sender)
{
  switch(VictorFSM1->State) // 正在发送文件
   {
     case mfsPrepareSendFile  : PrepareSendFile (); break; // 准备发送文件
     case mfsSendFileInfoReq  : SendFileInfoReq (); break; // 发送文件信息
     case mfsSendFileInfoWait : SendFileInfoWait(); break; // 发送文件信息：等待对方回应
     case mfsSendFileDataReq  : SendFileDataReq (); break; // 发送文件数据
     case mfsSendFileDataWait : SendFileDataWait(); break; // 发送文件数据：等待对方回应
     case mfsSendFileDone     : SendFileDone    (); break; // 发送文件结束
   }

  if(_bReceivingFile) // 正在接收文件
   {
     if(_bReceivingFile.Elapsed>5000)  // 超过 5000ms 没有收到数据
      {
        _bReceivingFile = false; // 取消接收文件
        if(_hReceiveFile!=INVALID_HANDLE_VALUE)  // 文件处于打开状态
         {
           CloseHandle(_hReceiveFile);           // 关闭文件
           _hReceiveFile = INVALID_HANDLE_VALUE; // 置为无效文件句柄
         }
        ShowInfo(L"接收文件超时，取消接收文件！");
      }
   }
}
//---------------------------------------------------------------------------
void TForm1::PrepareSendFile(void) // 准备发送文件 (mfsPrepareSendFile)
{
  if(_hSendFile!=INVALID_HANDLE_VALUE)
   {
     CloseHandle(_hSendFile); //关闭文件
     _hSendFile = INVALID_HANDLE_VALUE;
   }
  _hSendFile = CreateFile(_sUploadFileName.c_str(), GENERIC_READ, FILE_SHARE_READ|FILE_SHARE_WRITE, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
  if(_hSendFile==INVALID_HANDLE_VALUE)
   {
     VictorFSM1->State = mfsSendFileDone; // 发送文件结束
     ShowInfo(L"不能打开文件: "+_sUploadFileName);
     return;
   }
  _ucTryTimes = 0; // 重新尝试的次数清零
  VictorFSM1->State = mfsSendFileInfoReq; // 发送文件信息
  ShowInfo(L"发送文件: " + _sUploadFileName);
}
//---------------------------------------------------------------------------
void TForm1::SendFileInfoReq(void) // 发送文件信息 (mfsSendFileInfoReq)
{
  TMyFinfoPkg finf; //发送文件信息数据包

  finf.cmd  = mpcFileInfo;
  finf.id = ++_ucPkgID;
  wcscpy(finf.name,ExtractFileName(_sUploadFileName).c_str());

  _lUploadDataPos = 0;
  _lUploadDataSize = 0;
  _lUploadFileSize = SeekFile(_hSendFile,0,FILE_END); // SeekFile 的返回值是64位整数

  finf.size = _lUploadFileSize;                // 文件长度其实是64位整数，这里我们不处理大文件，所以只定义了32位的整数
  SeekFile(_hSendFile,0,FILE_BEGIN);

  finf.time = GetFileModifiedTime(_hSendFile);
  finf.CRC = ~CRC32(-1, &finf, sizeof(TMyFinfoPkg)-4);

  _ucTryTimes++; // 尝试次数
  YbCommDevice1->WritePackage(&finf, sizeof(TMyFinfoPkg));
  VictorFSM1->State = mfsSendFileInfoWait; // 发送文件信息 - 等待对方回应
}
//---------------------------------------------------------------------------
void TForm1::SendFileInfoWait(void) // 发送文件信息：等待对方回应 (mfsSendFileInfoWait)
{
  if(VictorFSM1->Elapsed>2000) // 处于这个状态的时间超过 2000ms 认为等待超时
   {
     if(_ucTryTimes<3)
      {
        VictorFSM1->State = mfsSendFileInfoReq;
        ShowInfo(L"等待超时：等待对方回应发送文件信息。\r\n第 "+IntToStr(_ucTryTimes)+" 次重新尝试。");
      }
     else
      {
        VictorFSM1->State = mfsSendFileDone; // 发送文件结束
        ShowInfo(L"等待超时：等待对方回应发送文件信息。\r\n发送文件已取消。");
      }
   }
}
//---------------------------------------------------------------------------
void TForm1::SendFileDataReq(void) // 发送文件数据 (mfsSendFileDataReq)
{
  TMyFdataPkg fdata;                                          // 发送文件内容数据包
  SeekFile(_hSendFile, _lUploadDataPos, FILE_BEGIN);                 // 从文件的第 _lUploadFilePos 个字节
  _lUploadDataSize = ReadFileData(_hSendFile, fdata.data, MyPkgDataSize); // 读取文件数据

  if(_lUploadDataSize>0) // 读取到文件的数据，需要发送
   {
     fdata.cmd = mpcFileData; // 发送文件数据
     fdata.from = _lUploadDataPos; // 从文件第 _lUploadFilePos 个字节开始发送
     fdata.size = _lUploadDataSize; // 发送文件数据的字节数
     fdata.id = ++_ucPkgID;
     fdata.CRC = ~CRC32(-1, &fdata, sizeof(TMyFdataPkg)-4);

     _ucTryTimes++; // 尝试次数
     YbCommDevice1->WritePackage(&fdata,sizeof(TMyFdataPkg));
     VictorFSM1->State = mfsSendFileDataWait; // 发送文件数据：等待对方回应
   }
  else // 没有更多的数据需要发送了
   {
     VictorFSM1->State = mfsSendFileDone; // 发送文件结束
     ShowInfo(L"文件所有的数据全部发送完成。");
   }
}
//---------------------------------------------------------------------------
void TForm1::SendFileDataWait(void) // 发送文件数据：等待对方回应 (mfsSendFileDataWait)
{
  if(VictorFSM1->Elapsed>2000) // 处于这个状态的时间超过 2000ms 认为等待超时
   {
     if(_ucTryTimes<3)
      {
        VictorFSM1->State = mfsSendFileDataReq;
        ShowInfo(L"等待超时：等待对方回应发送文件数据。\r\n第 "+IntToStr(_ucTryTimes)+" 次重新尝试。");
      }
     else
      {
        VictorFSM1->State = mfsSendFileDone; // 发送文件结束
        ShowInfo(L"等待超时：等待对方回应发送文件数据。\r\n发送文件已取消。");
      }
   }
}
//---------------------------------------------------------------------------
void TForm1::SendFileDone(void) // 发送文件结束 (mfsSendFileDone)
{
  if(_hSendFile!=INVALID_HANDLE_VALUE)
   {
     CloseHandle(_hSendFile); //关闭文件
     _hSendFile = INVALID_HANDLE_VALUE;
   }
  VictorFSM1->State = mfsIdle; // 空闲
}
//---------------------------------------------------------------------------
void __fastcall TForm1::YbCommDevice1Package(TObject *Sender, int NotifyType)
{
  if(NotifyType & EV_RXCHAR) // 收到数据包
   {
     typedef union
      {
        TMyFinfoPkg  finfo;   // 文件信息数据包
        TMyFdataPkg  fdata;   // 文件内容数据包
        TMyStatusPkg fstatus; // 对方来的回应数据包
        unsigned char cmd;    // 无论哪种数据包，第一个字节都是命令字节
      } TMyPackage;
     TMyPackage recv; // 收到的数据包可能是文件信息或者文件内容

     int nReceived; //收到的字节数
     while((nReceived=YbCommDevice1->ReadPackage(&recv,sizeof(TMyPackage))) > 0) // 收到数据包
      {
        switch(recv.cmd)
         {
           case mpcFileInfo:
                if(nReceived>=(int)sizeof(recv.finfo))
                  ReceivePackage(recv.finfo);
                break;
           case mpcFileData:
                if(nReceived>=(int)sizeof(recv.fdata))
                  ReceivePackage(recv.fdata);
                break;
           case mpcAnswer:
                if(nReceived>=(int)sizeof(recv.fstatus))
                  ReceivePackage(recv.fstatus);
                break;
         }
      }
   }
}
//---------------------------------------------------------------------------
void TForm1::ReceivePackage(TMyFinfoPkg &finfo) // 收到文件信息，对方请求传文件给我
{
  TMyStatusPkg send; //发送回应状态数据包
  send.cmd    = mpcAnswer;
  send.id     = finfo.id;
  send.anscmd = finfo.cmd;
  send.status = (finfo.CRC == ~CRC32(-1, &finfo, sizeof(finfo)-4)) ? mpeNoError : mpeCrcError;
  send.CRC = ~CRC32(-1, &send, sizeof(TMyStatusPkg)-4);

  if(send.status==mpeNoError) //数据 CRC 校验正确
   {
     _bReceivingFile = true; // 正在接收文件
     _lBytesToReceive = finfo.size;
     _dtReceiveFileTime = finfo.time;

     if(_hReceiveFile!=INVALID_HANDLE_VALUE)
      {
        CloseHandle(_hReceiveFile);
        _hReceiveFile = INVALID_HANDLE_VALUE;
      }

     UnicodeString sDestFileName;
     sDestFileName = _sReceivePath + finfo.name;
     _hReceiveFile = CreateFile(sDestFileName.c_str(), GENERIC_READ|GENERIC_WRITE, FILE_SHARE_READ, NULL, CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL);
     ShowInfo(L"接收文件: " + sDestFileName);

     if(_hReceiveFile==INVALID_HANDLE_VALUE) // 创建文件失败
      {
        send.status = mpeCreateFileError; // 创建文件错误
        _bReceivingFile = false; // 接收完成
        ShowInfo(L"无法创建文件: " + sDestFileName);
      }
     else if(_lBytesToReceive<=0) // 文件字节数为 0
      {
        SetFileModifiedTime(_hReceiveFile,_dtReceiveFileTime);
        CloseHandle(_hReceiveFile);
        _hReceiveFile = INVALID_HANDLE_VALUE;
        _bReceivingFile = false; // 接收完成
        ShowInfo(L"接收完成！");
      }
   }
  YbCommDevice1->WritePackage(&send,sizeof(TMyStatusPkg));
}
//---------------------------------------------------------------------------
void TForm1::ReceivePackage(TMyFdataPkg &fdata)
{
  TMyStatusPkg send; //发送回应状态数据包
  send.cmd    = mpcAnswer;
  send.id     = fdata.id;
  send.anscmd = fdata.cmd;
  send.status = mpeNoError;
  send.CRC    = ~CRC32(-1, &send, sizeof(TMyStatusPkg)-4);

  if(fdata.CRC != ~CRC32(-1, &fdata, sizeof(fdata)-4)) // CRC 错误
    send.status = mpeCrcError; // CRC 错误
  else if(_hReceiveFile==INVALID_HANDLE_VALUE) // 文件已关闭，是因为超时取消
    send.status = mpeTimeoutCanceled; // 超时取消

  if(send.status==mpeNoError) // 没有错误
   {
     _bReceivingFile = true; // 正在接收文件
     SeekFile(_hReceiveFile, fdata.from, FILE_BEGIN);
     WriteFileData(_hReceiveFile, fdata.data, fdata.size);
     if(fdata.from + fdata.size >= _lBytesToReceive)
      {
        SetFileModifiedTime(_hReceiveFile,_dtReceiveFileTime);
        CloseHandle(_hReceiveFile);
        _hReceiveFile = INVALID_HANDLE_VALUE;
        _bReceivingFile = false; // 接收完成
        ShowInfo(L"接收完成！");
      }
     ProgressBar1->Value = MakePercent(fdata.from + fdata.size, _lBytesToReceive);
   }
  YbCommDevice1->WritePackage(&send, sizeof(TMyStatusPkg));
}
//---------------------------------------------------------------------------
void TForm1::ReceivePackage(TMyStatusPkg &fstatus) // 收到对方的回应
{
  if(fstatus.id==_ucPkgID) // 对方回应的是我询问的命令
   {
     switch(fstatus.status) // 对方回应的错误编码
      {
        case mpeNoError: // 没有错误
             _ucTryTimes = 0; // 重新尝试的次数清零
             _lUploadDataPos += _lUploadDataSize; // 下一次从这个位置开始发送文件数据
             VictorFSM1->State = mfsSendFileDataReq; // 发送文件数据
             ProgressBar1->Value = MakePercent(_lUploadDataPos, _lUploadFileSize);
             break;
        case mpeTimeoutCanceled: // 超时取消
             VictorFSM1->State = mfsSendFileDone; // 发送文件结束
             ShowInfo(L"对方超时取消，发送文件结束。");
             break;
        case mpeCreateFileError: // 创建文件错误
             VictorFSM1->State = mfsSendFileDone; // 发送文件结束
             ShowInfo(L"对方创建文件错误，发送文件结束。");
             break;
        case mpeCrcError: // CRC 错误
             if(_ucTryTimes<3)
              {
                if(VictorFSM1->State==mfsSendFileInfoWait)
                 {
                   VictorFSM1->State = mfsSendFileInfoReq; // 重新尝试: 发送文件信息
                   ShowInfo(L"CRC 错误，重新尝试发送文件信息……");
                 }
                else if(VictorFSM1->State==mfsSendFileDataWait)
                 {
                   VictorFSM1->State = mfsSendFileDataReq; // 重新尝试: 发送文件数据
                   ShowInfo(L"CRC 错误，重新尝试发送文件数据……");
                 }
              }
             else
              {
                VictorFSM1->State = mfsSendFileDone; // 发送文件结束
                ShowInfo(L"CRC 错误，发送文件结束。");
              }
             break;
      }
   }
}
//---------------------------------------------------------------------------
void __fastcall TForm1::ShowErrDlg(const UnicodeString &sErrMsg)
{
#if(__BORLANDC__>=0x0720) // 0x0720: C++Builder 10.1 Berlin
  TDialogServiceSync::MessageDialog(sErrMsg, TMsgDlgType::mtError, TMsgDlgButtons()<<TMsgDlgBtn::mbOK, TMsgDlgBtn::mbOK, 0);
#else
  MessageDlg(sErrMsg, TMsgDlgType::mtError, TMsgDlgButtons()<<TMsgDlgBtn::mbOK, 0, TMsgDlgBtn::mbOK);
#endif
}
//---------------------------------------------------------------------------