﻿//---------------------------------------------------------------------------
// 串口控件演示程序，Copyright © Victor Chen

#ifndef UnitTsDataH
#define UnitTsDataH
//---------------------------------------------------------------------------
#include <vcl.h>
//---------------------------------------------------------------------------
namespace VictorData {
//---------------------------------------------------------------------------
enum TMyFsmState : int
{
  mfsIdle             , // 空闲
  mfsPrepareSendFile  , // 准备发送文件
  mfsSendFileInfoReq  , // 发送文件信息
  mfsSendFileInfoWait , // 发送文件信息：等待对方回应
  mfsSendFileDataReq  , // 发送文件数据
  mfsSendFileDataWait , // 发送文件数据：等待对方回应
  mfsSendFileDone     , // 发送文件结束
};
//---------------------------------------------------------------------------
const int MyPkgDataSize = 4096;
//---------------------------------------------------------------------------
enum TMyPkgCmd : unsigned char
{
  mpcFileInfo = 1,
  mpcFileData = 2,
  mpcAnswer   = 3,
};
//---------------------------------------------------------------------------
enum TMyPkgError : unsigned char
{
  mpeNoError          = 0x88, // 没有错误
  mpeTimeoutCanceled  = 0xcc, // 超时取消
  mpeCreateFileError  = 0xcf, // 创建文件错误
  mpeCrcError         = 0xee, // CRC 错误
};
//---------------------------------------------------------------------------
#pragma pack(push,1)
typedef struct
{
  unsigned char cmd;            // 1 (1: upload, 2: filedata, 3:status)
  unsigned char id;
  double        time;           // date & time
  long          size;           // filesize // 文件长度其实是64位整数，这里我们不处理大文件，所以只定义了32位的整数
  wchar_t       name[MAX_PATH]; // filename
  unsigned long CRC;
} TMyFinfoPkg;

typedef struct
{
  unsigned char cmd;   // 2 (1: upload, 2: filedata, 3:status)
  unsigned char id;
  long          from;  // 文件长度其实是64位整数，这里我们不处理大文件，所以只定义了32位的整数
  long          size;  // 文件长度其实是64位整数，这里我们不处理大文件，所以只定义了32位的整数
  char          data[MyPkgDataSize];
  unsigned long CRC;
} TMyFdataPkg;

typedef struct
{
  unsigned char cmd;     // 3
  unsigned char id;
  unsigned char anscmd;
  unsigned char status;  // 0x88: OK
  unsigned long CRC;
} TMyStatusPkg;
#pragma pack(pop)
//---------------------------------------------------------------------------
extern unsigned long CRC32(unsigned long c, void *s, int n);
extern int MakePercent(unsigned long lPos, unsigned long lCount);
//---------------------------------------------------------------------------
extern long long SeekFile(HANDLE hFile, long long llDistanceToMove, DWORD dwMoveMethod); // 文件长度和读写位置指针都是64位整数
extern DWORD ReadFileData(HANDLE hFile, void *ptr, DWORD n);
extern DWORD WriteFileData(HANDLE hFile, const void *ptr, DWORD n);
extern TDateTime GetFileModifiedTime(HANDLE hFile);
extern void SetFileModifiedTime(HANDLE hFile, TDateTime t);
//---------------------------------------------------------------------------
} using namespace VictorData;
//---------------------------------------------------------------------------
#endif
//---------------------------------------------------------------------------
